@@ -16,7 +16,7 @@
 ##' (\emph{a random number by default})
 ##' @param models a \code{vector} containing model names to be computed, must be among \code{GLM}, 
 ##' \code{GBM}, \code{GAM}, \code{CTA}, \code{ANN}, \code{SRE}, \code{FDA}, \code{MARS}, 
-##' \code{RF}, \code{MAXENT}, \code{MAXNET}
+##' \code{RF}, \code{MAXENT}, \code{MAXNET}, \code{XGBOOST}
 ##' @param models.pa (\emph{optional, default} \code{NULL}) \cr 
 ##' A \code{list} containing for each model a \code{vector} defining which pseudo-absence datasets 
 ##' are to be used, must be among \code{colnames(bm.format@PA.table)}
@@ -127,6 +127,7 @@
 ##'     \item \code{MAXENT} : Maximum Entropy 
 ##'     (\url{https://biodiversityinformatics.amnh.org/open_source/maxent/})
 ##'     \item \code{MAXNET} : Maximum Entropy (\code{\link[maxnet]{maxnet}})
+##'     \item \code{XGBOOST} : eXtreme Gradient Boosting Training (\code{\link[xgboost]{xgboost}})
 ##'   }}
 ##'   
 ##'   \item{models.pa}{Different models might respond differently to different numbers of 
@@ -188,15 +189,17 @@
 ##' @keywords models regression nonlinear multivariate nonparametric tree
 ##' 
 ##' 
-##' @seealso \code{\link[stats]{glm}}, \code{\link[gam]{gam}}, \code{\link[mgcv]{gam}}, 
-##' \code{\link[mgcv]{bam}}, \code{\link[gbm]{gbm}}, \code{\link[rpart]{rpart}}, 
-##' code{\link[nnet]{nnet}}, \code{\link[mda]{fda}}, \code{\link[earth]{earth}}, 
-##' \code{\link[randomForest]{randomForest}}, \code{\link[maxnet]{maxnet}},
-##' \code{\link{BIOMOD_FormatingData}}, \code{\link{BIOMOD_ModelingOptions}}, 
-##' \code{\link{bm_CrossValidation}}, \code{ \link{bm_VariablesImportance}}, 
-##' \code{\link{BIOMOD_Projection}}, \code{\link{BIOMOD_EnsembleModeling}},
-##' \code{\link{bm_PlotEvalMean}}, \code{\link{bm_PlotEvalBoxplot}}, 
-##' \code{\link{bm_PlotVarImpBoxplot}}, \code{\link{bm_PlotResponseCurves}}
+##' @seealso \code{\link[stats]{glm}}, \code{\link[gam]{gam}},
+##'   \code{\link[mgcv]{gam}}, \code{\link[mgcv]{bam}}, \code{\link[gbm]{gbm}},
+##'   \code{\link[rpart]{rpart}}, code{\link[nnet]{nnet}},
+##'   \code{\link[mda]{fda}}, \code{\link[earth]{earth}},
+##'   \code{\link[randomForest]{randomForest}}, \code{\link[maxnet]{maxnet}},
+##'   \code{\link[xgboost]{xgboost}}, \code{\link{BIOMOD_FormatingData}},
+##'   \code{\link{BIOMOD_ModelingOptions}}, \code{\link{bm_CrossValidation}},
+##'   \code{ \link{bm_VariablesImportance}}, \code{\link{BIOMOD_Projection}},
+##'   \code{\link{BIOMOD_EnsembleModeling}}, \code{\link{bm_PlotEvalMean}},
+##'   \code{\link{bm_PlotEvalBoxplot}}, \code{\link{bm_PlotVarImpBoxplot}},
+##'   \code{\link{bm_PlotResponseCurves}}
 ##' @family Main functions
 ##' 
 ##'   
@@ -286,7 +289,8 @@
 BIOMOD_Modeling <- function(bm.format,
                             modeling.id = as.character(format(Sys.time(), "%s")),
                             models = c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE'
-                                       , 'FDA', 'MARS', 'RF', 'MAXENT', 'MAXNET'),
+                                       , 'FDA', 'MARS', 'RF', 'MAXENT', 'MAXNET',
+                                       'XGBOOST'),
                             models.pa = NULL,
                             bm.options = NULL,
                             CV.strategy = 'random',
@@ -297,7 +301,7 @@ BIOMOD_Modeling <- function(bm.format,
                             CV.env.var = NULL,
                             CV.strat = NULL,
                             CV.user.table = NULL,
-                            CV.do.full.models = TRUE,
+                            CV.do.full.models = FALSE,
                             nb.rep,
                             data.split.perc,
                             data.split.table,
@@ -492,7 +496,7 @@ BIOMOD_Modeling <- function(bm.format,
   
   ## check if model is supported
   avail.models.list <- c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE', 'FDA', 'MARS'
-                         , 'RF', 'MAXENT', 'MAXNET')
+                         , 'RF', 'MAXENT', 'MAXNET', 'XGBOOST')
   .fun_testIfIn(TRUE, "models", models, avail.models.list)
   
   
@@ -500,7 +504,7 @@ BIOMOD_Modeling <- function(bm.format,
   categorical_var <- .get_categorical_names(bm.format@data.env.var)
   
   if (length(categorical_var) > 0) {
-    models.fact.unsupport <- c("SRE", "MAXENT.Tsuruoka")
+    models.fact.unsupport <- c("SRE", "XGBOOST")
     models.switch.off <- c(models.switch.off, intersect(models, models.fact.unsupport))
     if (length(models.switch.off) > 0) {
       models <- setdiff(models, models.switch.off)
